@@ -5,6 +5,8 @@
 #include "esphome/core/helpers.h"
 #include "esphome/components/binary_sensor/filter.h"
 
+#include <vector>
+
 namespace esphome {
 
 namespace binary_sensor {
@@ -17,6 +19,15 @@ namespace binary_sensor {
     } \
   }
 
+#define SUB_BINARY_SENSOR(name) \
+ protected: \
+  binary_sensor::BinarySensor *name##_binary_sensor_{nullptr}; \
+\
+ public: \
+  void set_##name##_binary_sensor(binary_sensor::BinarySensor *binary_sensor) { \
+    this->name##_binary_sensor_ = binary_sensor; \
+  }
+
 /** Base class for all binary_sensor-type classes.
  *
  * This class includes a callback that components such as MQTT can subscribe to for state changes.
@@ -58,6 +69,8 @@ class BinarySensor : public EntityBase {
   void add_filter(Filter *filter);
   void add_filters(const std::vector<Filter *> &filters);
 
+  void set_publish_initial_state(bool publish_initial_state) { this->publish_initial_state_ = publish_initial_state; }
+
   // ========== INTERNAL METHODS ==========
   // (In most use cases you won't need these)
   void send_state_internal(bool state, bool is_initial);
@@ -76,12 +89,11 @@ class BinarySensor : public EntityBase {
   virtual std::string device_class();
 
  protected:
-  uint32_t hash_base() override;
-
   CallbackManager<void(bool)> state_callback_{};
   optional<std::string> device_class_{};  ///< Stores the override of the device class
   Filter *filter_list_{nullptr};
   bool has_state_{false};
+  bool publish_initial_state_{false};
   Deduplicator<bool> publish_dedup_;
 };
 
