@@ -22,6 +22,7 @@ CONFIG_SCHEMA = cv.All(
         }
     ).extend(cv.COMPONENT_SCHEMA),
     cv.only_with_arduino,
+    cv.only_on(["esp32", "esp8266"]),
 )
 
 
@@ -36,3 +37,5 @@ async def to_code(config):
     if CORE.is_esp32:
         cg.add_library("DNSServer", None)
         cg.add_library("WiFi", None)
+    if CORE.is_esp8266:
+        cg.add_library("DNSServer", None)
