@@ -42,5 +42,19 @@ template<typename... Ts> class ControlAction : public Action<Ts...> {
   Climate *climate_;
 };
 
+class ControlTrigger : public Trigger<> {
+ public:
+  ControlTrigger(Climate *climate) {
+    climate->add_on_control_callback([this]() { this->trigger(); });
+  }
+};
+
+class StateTrigger : public Trigger<> {
+ public:
+  StateTrigger(Climate *climate) {
+    climate->add_on_state_callback([this]() { this->trigger(); });
+  }
+};
+
 }  // namespace climate
 }  // namespace esphome
