@@ -1,90 +1,32 @@
-import re
 import logging
 from pathlib import Path
 
 import esphome.config_validation as cv
+from esphome import git, loader
 from esphome.const import (
     CONF_COMPONENTS,
+    CONF_EXTERNAL_COMPONENTS,
+    CONF_PASSWORD,
+    CONF_PATH,
     CONF_REF,
     CONF_REFRESH,
     CONF_SOURCE,
-    CONF_URL,
     CONF_TYPE,
-    CONF_EXTERNAL_COMPONENTS,
-    CONF_PATH,
+    CONF_URL,
     CONF_USERNAME,
-    CONF_PASSWORD,
+    TYPE_GIT,
+    TYPE_LOCAL,
 )
 from esphome.core import CORE
-from esphome import git, loader
 
 _LOGGER = logging.getLogger(__name__)
 
 DOMAIN = CONF_EXTERNAL_COMPONENTS
 
-TYPE_GIT = "git"
-TYPE_LOCAL = "local"
-
-
-GIT_SCHEMA = {
-    cv.Required(CONF_URL): cv.url,
-    cv.Optional(CONF_REF): cv.git_ref,
-    cv.Optional(CONF_USERNAME): cv.string,
-    cv.Optional(CONF_PASSWORD): cv.string,
-}
-LOCAL_SCHEMA = {
-    cv.Required(CONF_PATH): cv.directory,
-}
-
-
-def validate_source_shorthand(value):
-    if not isinstance(value, str):
-        raise cv.Invalid("Shorthand only for strings")
-    try:
-        return SOURCE_SCHEMA({CONF_TYPE: TYPE_LOCAL, CONF_PATH: value})
-    except cv.Invalid:
-        pass
-    # Regex for GitHub repo name with optional branch/tag
-    # Note: git allows other branch/tag names as well, but never seen them used before
-    m = re.match(
-        r"github://(?:([a-zA-Z0-9\-]+)/([a-zA-Z0-9\-\._]+)(?:@([a-zA-Z0-9\-_.\./]+))?|pr#([0-9]+))",
-        value,
-    )
-    if m is None:
-        raise cv.Invalid(
-            "Source is not a file system path, in expected github://username/name[@branch-or-tag] or github://pr#1234 format!"
-        )
-    if m.group(4):
-        conf = {
-            CONF_TYPE: TYPE_GIT,
-            CONF_URL: "https://github.com/esphome/esphome.git",
-            CONF_REF: f"pull/{m.group(4)}/head",
-        }
-    else:
-        conf = {
-            CONF_TYPE: TYPE_GIT,
-            CONF_URL: f"https://github.com/{m.group(1)}/{m.group(2)}.git",
-        }
-        if m.group(3):
-            conf[CONF_REF] = m.group(3)
-
-    return SOURCE_SCHEMA(conf)
-
-
-SOURCE_SCHEMA = cv.Any(
-    validate_source_shorthand,
-    cv.typed_schema(
-        {
-            TYPE_GIT: cv.Schema(GIT_SCHEMA),
-            TYPE_LOCAL: cv.Schema(LOCAL_SCHEMA),
-        }
-    ),
-)
-
 
 CONFIG_SCHEMA = cv.ensure_list(
     {
-        cv.Required(CONF_SOURCE): SOURCE_SCHEMA,
+        cv.Required(CONF_SOURCE): cv.SOURCE_SCHEMA,
         cv.Optional(CONF_REFRESH, default="1d"): cv.All(cv.string, cv.source_refresh),
         cv.Optional(CONF_COMPONENTS, default="all"): cv.Any(
             "all", cv.ensure_list(cv.string)
