@@ -1,9 +1,25 @@
 #pragma once
 
+#include <cstdarg>
+#include <vector>
+#include "esphome/core/automation.h"
 #include "esphome/core/component.h"
-#include "esphome/core/log.h"
-#include "esphome/core/helpers.h"
 #include "esphome/core/defines.h"
+#include "esphome/core/helpers.h"
+
+#ifdef USE_ARDUINO
+#if defined(USE_ESP8266) || defined(USE_ESP32)
+#include <HardwareSerial.h>
+#endif  // USE_ESP8266 || USE_ESP32
+#ifdef USE_RP2040
+#include <HardwareSerial.h>
+#include <SerialUSB.h>
+#endif  // USE_RP2040
+#endif  // USE_ARDUINO
+
+#ifdef USE_ESP_IDF
+#include <driver/uart.h>
+#endif  // USE_ESP_IDF
 
 namespace esphome {
 
@@ -16,23 +32,42 @@ namespace logger {
 enum UARTSelection {
   UART_SELECTION_UART0 = 0,
   UART_SELECTION_UART1,
-#ifdef ARDUINO_ARCH_ESP32
-  UART_SELECTION_UART2
-#endif
-#ifdef ARDUINO_ARCH_ESP8266
-      UART_SELECTION_UART0_SWAP
-#endif
+#if defined(USE_ESP32)
+#if !defined(USE_ESP32_VARIANT_ESP32C3) && !defined(USE_ESP32_VARIANT_ESP32S2) && !defined(USE_ESP32_VARIANT_ESP32S3)
+  UART_SELECTION_UART2,
+#endif  // !USE_ESP32_VARIANT_ESP32C3 && !USE_ESP32_VARIANT_ESP32S2 && !USE_ESP32_VARIANT_ESP32S3
+#ifdef USE_ESP_IDF
+#if defined(USE_ESP32_VARIANT_ESP32S2) || defined(USE_ESP32_VARIANT_ESP32S3)
+  UART_SELECTION_USB_CDC,
+#endif  // USE_ESP32_VARIANT_ESP32S2 || USE_ESP32_VARIANT_ESP32S3
+#if defined(USE_ESP32_VARIANT_ESP32C3) || defined(USE_ESP32_VARIANT_ESP32S3)
+  UART_SELECTION_USB_SERIAL_JTAG,
+#endif  // USE_ESP32_VARIANT_ESP32C3 || USE_ESP32_VARIANT_ESP32S3
+#endif  // USE_ESP_IDF
+#endif  // USE_ESP32
+#ifdef USE_ESP8266
+  UART_SELECTION_UART0_SWAP,
+#endif  // USE_ESP8266
+#ifdef USE_RP2040
+  UART_SELECTION_USB_CDC,
+#endif  // USE_RP2040
 };
 
 class Logger : public Component {
  public:
-  explicit Logger(uint32_t baud_rate, size_t tx_buffer_size, UARTSelection uart);
+  explicit Logger(uint32_t baud_rate, size_t tx_buffer_size);
 
   /// Manually set the baud rate for serial, set to 0 to disable.
   void set_baud_rate(uint32_t baud_rate);
   uint32_t get_baud_rate() const { return baud_rate_; }
-  HardwareSerial *get_hw_serial() const { return hw_serial_; }
+#ifdef USE_ARDUINO
+  Stream *get_hw_serial() const { return hw_serial_; }
+#endif
+#ifdef USE_ESP_IDF
+  uart_port_t get_uart_num() const { return uart_num_; }
+#endif
 
+  void set_uart_selection(UARTSelection uart_selection) { uart_ = uart_selection; }
   /// Get the UART used by the logger.
   UARTSelection get_uart() const;
 
@@ -105,16 +140,38 @@ class Logger : public Component {
   int tx_buffer_at_{0};
   int tx_buffer_size_{0};
   UARTSelection uart_{UART_SELECTION_UART0};
-  HardwareSerial *hw_serial_{nullptr};
+#ifdef USE_ARDUINO
+  Stream *hw_serial_{nullptr};
+#endif
+#ifdef USE_ESP_IDF
+  uart_port_t uart_num_;
+#endif
   struct LogLevelOverride {
     std::string tag;
     int level;
   };
   std::vector<LogLevelOverride> log_levels_;
   CallbackManager<void(int, const char *, const char *)> log_callback_{};
+  /// Prevents recursive log calls, if true a log message is already being processed.
+  bool recursion_guard_ = false;
 };
 
-extern Logger *global_logger;
+extern Logger *global_logger;  // NOLINT(cppcoreguidelines-avoid-non-const-global-variables)
+
+class LoggerMessageTrigger : public Trigger<int, const char *, const char *> {
+ public:
+  explicit LoggerMessageTrigger(Logger *parent, int level) {
+    this->level_ = level;
+    parent->add_on_log_callback([this](int level, const char *tag, const char *message) {
+      if (level <= this->level_) {
+        this->trigger(level, tag, message);
+      }
+    });
+  }
+
+ protected:
+  int level_;
+};
 
 }  // namespace logger
 
