@@ -6,10 +6,8 @@ from esphome.const import (
     CONF_ID,
     CONF_OVERSAMPLING,
     CONF_RANGE,
-    DEVICE_CLASS_EMPTY,
     ICON_MAGNET,
     STATE_CLASS_MEASUREMENT,
-    STATE_CLASS_NONE,
     UNIT_MICROTESLA,
     UNIT_DEGREES,
     ICON_SCREEN_ROTATION,
@@ -71,10 +69,15 @@ def validate_enum(enum_values, units=None, int=True):
 
 
 field_strength_schema = sensor.sensor_schema(
-    UNIT_MICROTESLA, ICON_MAGNET, 1, DEVICE_CLASS_EMPTY, STATE_CLASS_MEASUREMENT
+    unit_of_measurement=UNIT_MICROTESLA,
+    icon=ICON_MAGNET,
+    accuracy_decimals=1,
+    state_class=STATE_CLASS_MEASUREMENT,
 )
 heading_schema = sensor.sensor_schema(
-    UNIT_DEGREES, ICON_SCREEN_ROTATION, 1, DEVICE_CLASS_EMPTY, STATE_CLASS_NONE
+    unit_of_measurement=UNIT_DEGREES,
+    icon=ICON_SCREEN_ROTATION,
+    accuracy_decimals=1,
 )
 
 CONFIG_SCHEMA = (
@@ -100,7 +103,7 @@ CONFIG_SCHEMA = (
 
 
 def auto_data_rate(config):
-    interval_sec = config[CONF_UPDATE_INTERVAL].seconds
+    interval_sec = config[CONF_UPDATE_INTERVAL].total_milliseconds / 1000
     interval_hz = 1.0 / interval_sec
     for datarate in sorted(QMC5883LDatarates.keys()):
         if float(datarate) >= interval_hz:
