@@ -25,9 +25,7 @@ CONFIG_SCHEMA = remote_base.validate_triggers(
     cv.Schema(
         {
             cv.GenerateID(): cv.declare_id(RemoteReceiverComponent),
-            cv.Required(CONF_PIN): cv.All(
-                pins.internal_gpio_input_pin_schema, pins.validate_has_interrupt
-            ),
+            cv.Required(CONF_PIN): cv.All(pins.internal_gpio_input_pin_schema),
             cv.Optional(CONF_DUMP, default=[]): remote_base.validate_dumpers,
             cv.Optional(CONF_TOLERANCE, default=25): cv.All(
                 cv.percentage_int, cv.Range(min=0)
@@ -58,7 +56,9 @@ async def to_code(config):
     for dumper in dumpers:
         cg.add(var.register_dumper(dumper))
 
-    await remote_base.build_triggers(config)
+    triggers = await remote_base.build_triggers(config)
+    for trigger in triggers:
+        cg.add(var.register_listener(trigger))
     await cg.register_component(var, config)
 
     cg.add(var.set_tolerance(config[CONF_TOLERANCE]))
