@@ -17,11 +17,11 @@ SafeModeButton = safe_mode_ns.class_("SafeModeButton", button.Button, cg.Compone
 
 CONFIG_SCHEMA = (
     button.button_schema(
+        SafeModeButton,
         device_class=DEVICE_CLASS_RESTART,
         entity_category=ENTITY_CATEGORY_CONFIG,
         icon=ICON_RESTART_ALERT,
     )
-    .extend({cv.GenerateID(): cv.declare_id(SafeModeButton)})
     .extend({cv.GenerateID(CONF_OTA): cv.use_id(OTAComponent)})
     .extend(cv.COMPONENT_SCHEMA)
 )
