@@ -3,6 +3,7 @@
 #include "esphome/core/automation.h"
 #include "esphome/core/component.h"
 #include "esphome/core/hal.h"
+#include "esphome/components/number/number.h"
 #include "esphome/components/switch/switch.h"
 
 #include <vector>
@@ -10,6 +11,8 @@
 namespace esphome {
 namespace sprinkler {
 
+const std::string min_str = "min";
+
 enum SprinklerState : uint8_t {
   // NOTE: these states are used by both SprinklerValveOperator and Sprinkler (the controller)!
   IDLE,      // system/valve is off
@@ -24,7 +27,14 @@ enum SprinklerTimerIndex : uint8_t {
   TIMER_VALVE_SELECTION = 1,
 };
 
+enum SprinklerValveRunRequestOrigin : uint8_t {
+  USER,
+  CYCLE,
+  QUEUE,
+};
+
 class Sprinkler;                  // this component
+class SprinklerControllerNumber;  // number components that appear in the front end; based on number core
 class SprinklerControllerSwitch;  // switches that appear in the front end; based on switch core
 class SprinklerSwitch;            // switches representing any valve or pump; provides abstraction for latching valves
 class SprinklerValveOperator;     // manages all switching on/off of valves and associated pumps
@@ -76,6 +86,7 @@ struct SprinklerTimer {
 };
 
 struct SprinklerValve {
+  SprinklerControllerNumber *run_duration_number;
   SprinklerControllerSwitch *controller_switch;
   SprinklerControllerSwitch *enable_switch;
   SprinklerSwitch valve_switch;
@@ -88,6 +99,25 @@ struct SprinklerValve {
   std::unique_ptr<Automation<>> valve_turn_on_automation;
 };
 
+class SprinklerControllerNumber : public number::Number, public Component {
+ public:
+  void setup() override;
+  void dump_config() override;
+  float get_setup_priority() const override { return setup_priority::PROCESSOR; }
+
+  Trigger<float> *get_set_trigger() const { return set_trigger_; }
+  void set_initial_value(float initial_value) { initial_value_ = initial_value; }
+  void set_restore_value(bool restore_value) { this->restore_value_ = restore_value; }
+
+ protected:
+  void control(float value) override;
+  float initial_value_{NAN};
+  bool restore_value_{true};
+  Trigger<float> *set_trigger_ = new Trigger<float>();
+
+  ESPPreferenceObject pref_;
+};
+
 class SprinklerControllerSwitch : public switch_::Switch, public Component {
  public:
   SprinklerControllerSwitch();
@@ -96,27 +126,19 @@ class SprinklerControllerSwitch : public switch_::Switch, public Component {
   void dump_config() override;
 
   void set_state_lambda(std::function<optional<bool>()> &&f);
-  void set_restore_state(bool restore_state);
   Trigger<> *get_turn_on_trigger() const;
   Trigger<> *get_turn_off_trigger() const;
-  void set_optimistic(bool optimistic);
-  void set_assumed_state(bool assumed_state);
   void loop() override;
 
   float get_setup_priority() const override;
 
  protected:
-  bool assumed_state() override;
-
   void write_state(bool state) override;
 
   optional<std::function<optional<bool>()>> f_;
-  bool optimistic_{false};
-  bool assumed_state_{false};
   Trigger<> *turn_on_trigger_;
   Trigger<> *turn_off_trigger_;
   Trigger<> *prev_trigger_{nullptr};
-  bool restore_state_{false};
 };
 
 class SprinklerValveOperator {
@@ -160,6 +182,7 @@ class SprinklerValveRunRequest {
   SprinklerValveRunRequest(size_t valve_number, uint32_t run_duration, SprinklerValveOperator *valve_op);
   bool has_request();
   bool has_valve_operator();
+  void set_request_from(SprinklerValveRunRequestOrigin origin);
   void set_run_duration(uint32_t run_duration);
   void set_valve(size_t valve_number);
   void set_valve_operator(SprinklerValveOperator *valve_op);
@@ -168,12 +191,14 @@ class SprinklerValveRunRequest {
   size_t valve();
   optional<size_t> valve_as_opt();
   SprinklerValveOperator *valve_operator();
+  SprinklerValveRunRequestOrigin request_is_from();
 
  protected:
   bool has_valve_{false};
   size_t valve_number_{0};
   uint32_t run_duration_{0};
   SprinklerValveOperator *valve_op_{nullptr};
+  SprinklerValveRunRequestOrigin origin_{USER};
 };
 
 class Sprinkler : public Component, public EntityBase {
@@ -196,6 +221,11 @@ class Sprinkler : public Component, public EntityBase {
   void set_controller_auto_adv_switch(SprinklerControllerSwitch *auto_adv_switch);
   void set_controller_queue_enable_switch(SprinklerControllerSwitch *queue_enable_switch);
   void set_controller_reverse_switch(SprinklerControllerSwitch *reverse_switch);
+  void set_controller_standby_switch(SprinklerControllerSwitch *standby_switch);
+
+  /// configure important controller number components
+  void set_controller_multiplier_number(SprinklerControllerNumber *multiplier_number);
+  void set_controller_repeat_number(SprinklerControllerNumber *repeat_number);
 
   /// configure a valve's switch object and run duration. run_duration is time in seconds.
   void configure_valve_switch(size_t valve_number, switch_::Switch *valve_switch, uint32_t run_duration);
@@ -207,9 +237,18 @@ class Sprinkler : public Component, public EntityBase {
   void configure_valve_pump_switch_pulsed(size_t valve_number, switch_::Switch *pump_switch_off,
                                           switch_::Switch *pump_switch_on, uint32_t pulse_duration);
 
+  /// configure a valve's run duration number component
+  void configure_valve_run_duration_number(size_t valve_number, SprinklerControllerNumber *run_duration_number);
+
+  /// sets the multiplier value to '1 / divider' and sets repeat value to divider
+  void set_divider(optional<uint32_t> divider);
+
   /// value multiplied by configured run times -- used to extend or shorten the cycle
   void set_multiplier(optional<float> multiplier);
 
+  /// enable/disable skipping of disabled valves by the next and previous actions
+  void set_next_prev_ignore_disabled_valves(bool ignore_disabled);
+
   /// set how long the pump should start after the valve (when the pump is starting)
   void set_pump_start_delay(uint32_t start_delay);
 
@@ -250,6 +289,9 @@ class Sprinkler : public Component, public EntityBase {
   /// if reverse is true, controller will iterate through all enabled valves in reverse (descending) order
   void set_reverse(bool reverse);
 
+  /// if standby is true, controller will refuse to activate any valves
+  void set_standby(bool standby);
+
   /// returns valve_number's run duration in seconds
   uint32_t valve_run_duration(size_t valve_number);
 
@@ -274,6 +316,9 @@ class Sprinkler : public Component, public EntityBase {
   /// returns true if reverse is enabled
   bool reverse();
 
+  /// returns true if standby is enabled
+  bool standby();
+
   /// starts the controller from the first valve in the queue and disables auto_advance.
   /// if the queue is empty, does nothing.
   void start_from_queue();
@@ -283,7 +328,7 @@ class Sprinkler : public Component, public EntityBase {
   void start_full_cycle();
 
   /// activates a single valve and disables auto_advance.
-  void start_single_valve(optional<size_t> valve_number);
+  void start_single_valve(optional<size_t> valve_number, optional<uint32_t> run_duration = nullopt);
 
   /// adds a valve into the queue. queued valves have priority over valves to be run as a part of a full cycle.
   /// NOTE: queued valves will always run, regardless of auto-advance and/or valve enable switches.
@@ -316,6 +361,9 @@ class Sprinkler : public Component, public EntityBase {
   /// returns a pointer to a valve's name string object; returns nullptr if valve_number is invalid
   const char *valve_name(size_t valve_number);
 
+  /// returns what invoked the valve that is currently active, if any. check with 'has_value()'
+  optional<SprinklerValveRunRequestOrigin> active_valve_request_is_from();
+
   /// returns the number of the valve that is currently active, if any. check with 'has_value()'
   optional<size_t> active_valve();
 
@@ -341,8 +389,23 @@ class Sprinkler : public Component, public EntityBase {
   /// switches on/off a pump "safely" by checking that the new state will not conflict with another controller
   void set_pump_state(SprinklerSwitch *pump_switch, bool state);
 
-  /// returns the amount of time remaining in seconds for the active valve, if any. check with 'has_value()'
-  optional<uint32_t> time_remaining();
+  /// returns the amount of time in seconds required for all valves
+  uint32_t total_cycle_time_all_valves();
+
+  /// returns the amount of time in seconds required for all enabled valves
+  uint32_t total_cycle_time_enabled_valves();
+
+  /// returns the amount of time in seconds required for all enabled & incomplete valves, not including the active valve
+  uint32_t total_cycle_time_enabled_incomplete_valves();
+
+  /// returns the amount of time in seconds required for all valves in the queue
+  uint32_t total_queue_time();
+
+  /// returns the amount of time remaining in seconds for the active valve, if any
+  optional<uint32_t> time_remaining_active_valve();
+
+  /// returns the amount of time remaining in seconds for all valves remaining, including the active valve, if any
+  optional<uint32_t> time_remaining_current_operation();
 
   /// returns a pointer to a valve's control switch object
   SprinklerControllerSwitch *control_switch(size_t valve_number);
@@ -371,9 +434,13 @@ class Sprinkler : public Component, public EntityBase {
   /// returns true if valve's cycle is flagged as complete
   bool valve_cycle_complete_(size_t valve_number);
 
-  /// returns the number of the next/previous valve in the vector
-  size_t next_valve_number_(size_t first_valve);
-  size_t previous_valve_number_(size_t first_valve);
+  /// returns the number of the next valve in the vector or nullopt if no valves match criteria
+  optional<size_t> next_valve_number_(optional<size_t> first_valve = nullopt, bool include_disabled = true,
+                                      bool include_complete = true);
+
+  /// returns the number of the previous valve in the vector or nullopt if no valves match criteria
+  optional<size_t> previous_valve_number_(optional<size_t> first_valve = nullopt, bool include_disabled = true,
+                                          bool include_complete = true);
 
   /// returns the number of the next valve that should be activated in a full cycle.
   ///  if no valve is next (cycle is complete), returns no value (check with 'has_value()')
@@ -385,11 +452,6 @@ class Sprinkler : public Component, public EntityBase {
   ///  if no valve is next (for example, a full cycle is complete), next_req_ is reset via reset().
   void load_next_valve_run_request_(optional<size_t> first_valve = nullopt);
 
-  /// returns the number of the next/previous valve that should be activated.
-  ///  if no valve is next (cycle is complete), returns no value (check with 'has_value()')
-  optional<size_t> next_enabled_incomplete_valve_number_(optional<size_t> first_valve);
-  optional<size_t> previous_enabled_incomplete_valve_number_(optional<size_t> first_valve);
-
   /// returns true if any valve is enabled
   bool any_valve_is_enabled_();
 
@@ -424,7 +486,10 @@ class Sprinkler : public Component, public EntityBase {
   /// starts up the system from IDLE state
   void fsm_transition_to_shutdown_();
 
-  /// return the current FSM state as a string
+  /// return the specified SprinklerValveRunRequestOrigin as a string
+  std::string req_as_str_(SprinklerValveRunRequestOrigin origin);
+
+  /// return the specified SprinklerState state as a string
   std::string state_as_str_(SprinklerState state);
 
   /// Start/cancel/get status of valve timers
@@ -446,6 +511,9 @@ class Sprinkler : public Component, public EntityBase {
   /// Maximum allowed queue size
   const uint8_t max_queue_size_{100};
 
+  /// When set to true, the next and previous actions will skip disabled valves
+  bool next_prev_ignore_disabled_{false};
+
   /// Pump should be off during valve_open_delay interval
   bool pump_switch_off_during_valve_open_delay_{false};
 
@@ -518,12 +586,19 @@ class Sprinkler : public Component, public EntityBase {
   SprinklerControllerSwitch *controller_sw_{nullptr};
   SprinklerControllerSwitch *queue_enable_sw_{nullptr};
   SprinklerControllerSwitch *reverse_sw_{nullptr};
+  SprinklerControllerSwitch *standby_sw_{nullptr};
+
+  /// Number components we'll present to the front end
+  SprinklerControllerNumber *multiplier_number_{nullptr};
+  SprinklerControllerNumber *repeat_number_{nullptr};
 
   std::unique_ptr<ShutdownAction<>> sprinkler_shutdown_action_;
+  std::unique_ptr<ShutdownAction<>> sprinkler_standby_shutdown_action_;
   std::unique_ptr<ResumeOrStartAction<>> sprinkler_resumeorstart_action_;
 
   std::unique_ptr<Automation<>> sprinkler_turn_off_automation_;
   std::unique_ptr<Automation<>> sprinkler_turn_on_automation_;
+  std::unique_ptr<Automation<>> sprinkler_standby_turn_on_automation_;
 };
 
 }  // namespace sprinkler
