@@ -1,6 +1,6 @@
 """Constants used by esphome."""
 
-__version__ = "2023.2.4"
+__version__ = "2023.3.0"
 
 ALLOWED_NAME_CHARS = "abcdefghijklmnopqrstuvwxyz0123456789-_"
 
@@ -214,6 +214,7 @@ CONF_ENERGY = "energy"
 CONF_ENTITY_CATEGORY = "entity_category"
 CONF_ENTITY_ID = "entity_id"
 CONF_ENUM_DATAPOINT = "enum_datapoint"
+CONF_EQUATION = "equation"
 CONF_ESP8266_DISABLE_SSL_SUPPORT = "esp8266_disable_ssl_support"
 CONF_ESPHOME = "esphome"
 CONF_ETHERNET = "ethernet"
@@ -447,6 +448,7 @@ CONF_ON_BLE_SERVICE_DATA_ADVERTISE = "on_ble_service_data_advertise"
 CONF_ON_BOOT = "on_boot"
 CONF_ON_CLICK = "on_click"
 CONF_ON_CONNECT = "on_connect"
+CONF_ON_CONTROL = "on_control"
 CONF_ON_DISCONNECT = "on_disconnect"
 CONF_ON_DOUBLE_CLICK = "on_double_click"
 CONF_ON_ENROLLMENT_DONE = "on_enrollment_done"
@@ -797,6 +799,9 @@ CONF_X_GRID = "x_grid"
 CONF_Y_GRID = "y_grid"
 CONF_ZERO = "zero"
 
+TYPE_GIT = "git"
+TYPE_LOCAL = "local"
+
 ENV_NOGITIGNORE = "ESPHOME_NOGITIGNORE"
 ENV_QUICKWIZARD = "ESPHOME_QUICKWIZARD"
 
@@ -856,6 +861,7 @@ ICON_SIGNAL_DISTANCE_VARIANT = "mdi:signal"
 ICON_THERMOMETER = "mdi:thermometer"
 ICON_TIMELAPSE = "mdi:timelapse"
 ICON_TIMER = "mdi:timer-outline"
+ICON_WATER = "mdi:water"
 ICON_WATER_PERCENT = "mdi:water-percent"
 ICON_WEATHER_SUNSET = "mdi:weather-sunset"
 ICON_WEATHER_SUNSET_DOWN = "mdi:weather-sunset-down"
@@ -877,6 +883,7 @@ UNIT_DEGREE_PER_SECOND = "°/s"
 UNIT_DEGREES = "°"
 UNIT_EMPTY = ""
 UNIT_G = "G"
+UNIT_GRAMS_PER_CUBIC_METER = "g/m³"
 UNIT_HECTOPASCAL = "hPa"
 UNIT_HERTZ = "Hz"
 UNIT_HOUR = "h"
@@ -916,80 +923,87 @@ UNIT_VOLT_AMPS_REACTIVE_HOURS = "VARh"
 UNIT_WATT = "W"
 UNIT_WATT_HOURS = "Wh"
 
-# device classes of binary_sensor component
-DEVICE_CLASS_BATTERY_CHARGING = "battery_charging"
-DEVICE_CLASS_COLD = "cold"
-DEVICE_CLASS_CONNECTIVITY = "connectivity"
-DEVICE_CLASS_DOOR = "door"
-DEVICE_CLASS_GARAGE_DOOR = "garage_door"
-DEVICE_CLASS_HEAT = "heat"
-DEVICE_CLASS_LIGHT = "light"
-DEVICE_CLASS_LOCK = "lock"
-DEVICE_CLASS_MOTION = "motion"
-DEVICE_CLASS_MOVING = "moving"
-DEVICE_CLASS_OCCUPANCY = "occupancy"
-DEVICE_CLASS_OPENING = "opening"
-DEVICE_CLASS_PLUG = "plug"
-DEVICE_CLASS_PRESENCE = "presence"
-DEVICE_CLASS_PROBLEM = "problem"
-DEVICE_CLASS_RUNNING = "running"
-DEVICE_CLASS_SAFETY = "safety"
-DEVICE_CLASS_SMOKE = "smoke"
-DEVICE_CLASS_SOUND = "sound"
-DEVICE_CLASS_TAMPER = "tamper"
-DEVICE_CLASS_VIBRATION = "vibration"
-DEVICE_CLASS_WINDOW = "window"
-# device classes of both binary_sensor and sensor component
-DEVICE_CLASS_EMPTY = ""
-DEVICE_CLASS_BATTERY = "battery"
-DEVICE_CLASS_CARBON_MONOXIDE = "carbon_monoxide"
-DEVICE_CLASS_GAS = "gas"
-DEVICE_CLASS_MOISTURE = "moisture"
-DEVICE_CLASS_POWER = "power"
-# device classes of sensor component
+# device classes
 DEVICE_CLASS_APPARENT_POWER = "apparent_power"
 DEVICE_CLASS_AQI = "aqi"
+DEVICE_CLASS_ATMOSPHERIC_PRESSURE = "atmospheric_pressure"
+DEVICE_CLASS_AWNING = "awning"
+DEVICE_CLASS_BATTERY = "battery"
+DEVICE_CLASS_BATTERY_CHARGING = "battery_charging"
+DEVICE_CLASS_BLIND = "blind"
 DEVICE_CLASS_CARBON_DIOXIDE = "carbon_dioxide"
+DEVICE_CLASS_CARBON_MONOXIDE = "carbon_monoxide"
+DEVICE_CLASS_COLD = "cold"
+DEVICE_CLASS_CONNECTIVITY = "connectivity"
 DEVICE_CLASS_CURRENT = "current"
+DEVICE_CLASS_CURTAIN = "curtain"
+DEVICE_CLASS_DAMPER = "damper"
+DEVICE_CLASS_DATA_RATE = "data_rate"
+DEVICE_CLASS_DATA_SIZE = "data_size"
 DEVICE_CLASS_DATE = "date"
 DEVICE_CLASS_DISTANCE = "distance"
+DEVICE_CLASS_DOOR = "door"
 DEVICE_CLASS_DURATION = "duration"
+DEVICE_CLASS_EMPTY = ""
 DEVICE_CLASS_ENERGY = "energy"
 DEVICE_CLASS_FREQUENCY = "frequency"
+DEVICE_CLASS_GARAGE = "garage"
+DEVICE_CLASS_GARAGE_DOOR = "garage_door"
+DEVICE_CLASS_GAS = "gas"
+DEVICE_CLASS_GATE = "gate"
+DEVICE_CLASS_HEAT = "heat"
 DEVICE_CLASS_HUMIDITY = "humidity"
 DEVICE_CLASS_ILLUMINANCE = "illuminance"
+DEVICE_CLASS_IRRADIANCE = "irradiance"
+DEVICE_CLASS_LIGHT = "light"
+DEVICE_CLASS_LOCK = "lock"
+DEVICE_CLASS_MOISTURE = "moisture"
 DEVICE_CLASS_MONETARY = "monetary"
+DEVICE_CLASS_MOTION = "motion"
+DEVICE_CLASS_MOVING = "moving"
 DEVICE_CLASS_NITROGEN_DIOXIDE = "nitrogen_dioxide"
 DEVICE_CLASS_NITROGEN_MONOXIDE = "nitrogen_monoxide"
 DEVICE_CLASS_NITROUS_OXIDE = "nitrous_oxide"
+DEVICE_CLASS_OCCUPANCY = "occupancy"
+DEVICE_CLASS_OPENING = "opening"
+DEVICE_CLASS_OUTLET = "outlet"
 DEVICE_CLASS_OZONE = "ozone"
+DEVICE_CLASS_PLUG = "plug"
 DEVICE_CLASS_PM1 = "pm1"
 DEVICE_CLASS_PM10 = "pm10"
 DEVICE_CLASS_PM25 = "pm25"
+DEVICE_CLASS_POWER = "power"
 DEVICE_CLASS_POWER_FACTOR = "power_factor"
 DEVICE_CLASS_PRECIPITATION = "precipitation"
 DEVICE_CLASS_PRECIPITATION_INTENSITY = "precipitation_intensity"
+DEVICE_CLASS_PRESENCE = "presence"
 DEVICE_CLASS_PRESSURE = "pressure"
+DEVICE_CLASS_PROBLEM = "problem"
 DEVICE_CLASS_REACTIVE_POWER = "reactive_power"
+DEVICE_CLASS_RESTART = "restart"
+DEVICE_CLASS_RUNNING = "running"
+DEVICE_CLASS_SAFETY = "safety"
+DEVICE_CLASS_SHADE = "shade"
+DEVICE_CLASS_SHUTTER = "shutter"
 DEVICE_CLASS_SIGNAL_STRENGTH = "signal_strength"
+DEVICE_CLASS_SMOKE = "smoke"
+DEVICE_CLASS_SOUND = "sound"
+DEVICE_CLASS_SOUND_PRESSURE = "sound_pressure"
 DEVICE_CLASS_SPEED = "speed"
 DEVICE_CLASS_SULPHUR_DIOXIDE = "sulphur_dioxide"
+DEVICE_CLASS_SWITCH = "switch"
+DEVICE_CLASS_TAMPER = "tamper"
 DEVICE_CLASS_TEMPERATURE = "temperature"
 DEVICE_CLASS_TIMESTAMP = "timestamp"
+DEVICE_CLASS_UPDATE = "update"
+DEVICE_CLASS_VIBRATION = "vibration"
 DEVICE_CLASS_VOLATILE_ORGANIC_COMPOUNDS = "volatile_organic_compounds"
 DEVICE_CLASS_VOLTAGE = "voltage"
 DEVICE_CLASS_VOLUME = "volume"
 DEVICE_CLASS_WATER = "water"
-DEVICE_CLASS_WIND_SPEED = "wind_speed"
 DEVICE_CLASS_WEIGHT = "weight"
-# device classes of both binary_sensor and button component
-DEVICE_CLASS_UPDATE = "update"
-# device classes of button component
-DEVICE_CLASS_RESTART = "restart"
-# device classes of switch component
-DEVICE_CLASS_OUTLET = "outlet"
-DEVICE_CLASS_SWITCH = "switch"
-
+DEVICE_CLASS_WINDOW = "window"
+DEVICE_CLASS_WIND_SPEED = "wind_speed"
 
 # state classes
 STATE_CLASS_NONE = ""
@@ -1009,6 +1023,7 @@ KEY_TARGET_FRAMEWORK = "target_framework"
 KEY_FRAMEWORK_VERSION = "framework_version"
 KEY_NAME = "name"
 KEY_VARIANT = "variant"
+KEY_PAST_SAFE_MODE = "past_safe_mode"
 
 # Entity categories
 ENTITY_CATEGORY_NONE = ""
