@@ -2,10 +2,11 @@
 
 #include <string>
 #include <vector>
-#include "esphome/core/defines.h"
-#include "esphome/core/preferences.h"
 #include "esphome/core/component.h"
+#include "esphome/core/defines.h"
+#include "esphome/core/hal.h"
 #include "esphome/core/helpers.h"
+#include "esphome/core/preferences.h"
 #include "esphome/core/scheduler.h"
 
 #ifdef USE_BINARY_SENSOR
@@ -17,6 +18,9 @@
 #ifdef USE_SWITCH
 #include "esphome/components/switch/switch.h"
 #endif
+#ifdef USE_BUTTON
+#include "esphome/components/button/button.h"
+#endif
 #ifdef USE_TEXT_SENSOR
 #include "esphome/components/text_sensor/text_sensor.h"
 #endif
@@ -32,20 +36,40 @@
 #ifdef USE_COVER
 #include "esphome/components/cover/cover.h"
 #endif
+#ifdef USE_NUMBER
+#include "esphome/components/number/number.h"
+#endif
+#ifdef USE_SELECT
+#include "esphome/components/select/select.h"
+#endif
+#ifdef USE_LOCK
+#include "esphome/components/lock/lock.h"
+#endif
+#ifdef USE_MEDIA_PLAYER
+#include "esphome/components/media_player/media_player.h"
+#endif
 
 namespace esphome {
 
 class Application {
  public:
-  void pre_setup(const std::string &name, const char *compilation_time, bool name_add_mac_suffix) {
+  void pre_setup(const std::string &name, const std::string &friendly_name, const std::string &comment,
+                 const char *compilation_time, bool name_add_mac_suffix) {
+    arch_init();
     this->name_add_mac_suffix_ = name_add_mac_suffix;
     if (name_add_mac_suffix) {
       this->name_ = name + "-" + get_mac_address().substr(6);
+      if (friendly_name.empty()) {
+        this->friendly_name_ = "";
+      } else {
+        this->friendly_name_ = friendly_name + " " + get_mac_address().substr(6);
+      }
     } else {
       this->name_ = name;
+      this->friendly_name_ = friendly_name;
     }
+    this->comment_ = comment;
     this->compilation_time_ = compilation_time;
-    global_preferences.begin();
   }
 
 #ifdef USE_BINARY_SENSOR
@@ -62,12 +86,16 @@ class Application {
   void register_switch(switch_::Switch *a_switch) { this->switches_.push_back(a_switch); }
 #endif
 
+#ifdef USE_BUTTON
+  void register_button(button::Button *button) { this->buttons_.push_back(button); }
+#endif
+
 #ifdef USE_TEXT_SENSOR
   void register_text_sensor(text_sensor::TextSensor *sensor) { this->text_sensors_.push_back(sensor); }
 #endif
 
 #ifdef USE_FAN
-  void register_fan(fan::FanState *state) { this->fans_.push_back(state); }
+  void register_fan(fan::Fan *state) { this->fans_.push_back(state); }
 #endif
 
 #ifdef USE_COVER
@@ -82,6 +110,22 @@ class Application {
   void register_light(light::LightState *light) { this->lights_.push_back(light); }
 #endif
 
+#ifdef USE_NUMBER
+  void register_number(number::Number *number) { this->numbers_.push_back(number); }
+#endif
+
+#ifdef USE_SELECT
+  void register_select(select::Select *select) { this->selects_.push_back(select); }
+#endif
+
+#ifdef USE_LOCK
+  void register_lock(lock::Lock *a_lock) { this->locks_.push_back(a_lock); }
+#endif
+
+#ifdef USE_MEDIA_PLAYER
+  void register_media_player(media_player::MediaPlayer *media_player) { this->media_players_.push_back(media_player); }
+#endif
+
   /// Register the component in this Application instance.
   template<class C> C *register_component(C *c) {
     static_assert(std::is_base_of<Component, C>::value, "Only Component subclasses can be registered");
@@ -95,9 +139,14 @@ class Application {
   /// Make a loop iteration. Call this in your loop() function.
   void loop();
 
-  /// Get the name of this Application set by set_name().
+  /// Get the name of this Application set by pre_setup().
   const std::string &get_name() const { return this->name_; }
 
+  /// Get the friendly name of this Application set by pre_setup().
+  const std::string &get_friendly_name() const { return this->friendly_name_; }
+  /// Get the comment of this Application set by pre_setup().
+  const std::string &get_comment() const { return this->comment_; }
+
   bool is_name_add_mac_suffix_enabled() const { return this->name_add_mac_suffix_; }
 
   const std::string &get_compilation_time() const { return this->compilation_time_; }
@@ -125,14 +174,7 @@ class Application {
 
   void safe_reboot();
 
-  void run_safe_shutdown_hooks() {
-    for (auto *comp : this->components_) {
-      comp->on_safe_shutdown();
-    }
-    for (auto *comp : this->components_) {
-      comp->on_shutdown();
-    }
-  }
+  void run_safe_shutdown_hooks();
 
   uint32_t get_app_state() const { return this->app_state_; }
 
@@ -154,6 +196,15 @@ class Application {
     return nullptr;
   }
 #endif
+#ifdef USE_BUTTON
+  const std::vector<button::Button *> &get_buttons() { return this->buttons_; }
+  button::Button *get_button_by_key(uint32_t key, bool include_internal = false) {
+    for (auto *obj : this->buttons_)
+      if (obj->get_object_id_hash() == key && (include_internal || !obj->is_internal()))
+        return obj;
+    return nullptr;
+  }
+#endif
 #ifdef USE_SENSOR
   const std::vector<sensor::Sensor *> &get_sensors() { return this->sensors_; }
   sensor::Sensor *get_sensor_by_key(uint32_t key, bool include_internal = false) {
@@ -173,8 +224,8 @@ class Application {
   }
 #endif
 #ifdef USE_FAN
-  const std::vector<fan::FanState *> &get_fans() { return this->fans_; }
-  fan::FanState *get_fan_by_key(uint32_t key, bool include_internal = false) {
+  const std::vector<fan::Fan *> &get_fans() { return this->fans_; }
+  fan::Fan *get_fan_by_key(uint32_t key, bool include_internal = false) {
     for (auto *obj : this->fans_)
       if (obj->get_object_id_hash() == key && (include_internal || !obj->is_internal()))
         return obj;
@@ -208,6 +259,42 @@ class Application {
     return nullptr;
   }
 #endif
+#ifdef USE_NUMBER
+  const std::vector<number::Number *> &get_numbers() { return this->numbers_; }
+  number::Number *get_number_by_key(uint32_t key, bool include_internal = false) {
+    for (auto *obj : this->numbers_)
+      if (obj->get_object_id_hash() == key && (include_internal || !obj->is_internal()))
+        return obj;
+    return nullptr;
+  }
+#endif
+#ifdef USE_SELECT
+  const std::vector<select::Select *> &get_selects() { return this->selects_; }
+  select::Select *get_select_by_key(uint32_t key, bool include_internal = false) {
+    for (auto *obj : this->selects_)
+      if (obj->get_object_id_hash() == key && (include_internal || !obj->is_internal()))
+        return obj;
+    return nullptr;
+  }
+#endif
+#ifdef USE_LOCK
+  const std::vector<lock::Lock *> &get_locks() { return this->locks_; }
+  lock::Lock *get_lock_by_key(uint32_t key, bool include_internal = false) {
+    for (auto *obj : this->locks_)
+      if (obj->get_object_id_hash() == key && (include_internal || !obj->is_internal()))
+        return obj;
+    return nullptr;
+  }
+#endif
+#ifdef USE_MEDIA_PLAYER
+  const std::vector<media_player::MediaPlayer *> &get_media_players() { return this->media_players_; }
+  media_player::MediaPlayer *get_media_player_by_key(uint32_t key, bool include_internal = false) {
+    for (auto *obj : this->media_players_)
+      if (obj->get_object_id_hash() == key && (include_internal || !obj->is_internal()))
+        return obj;
+    return nullptr;
+  }
+#endif
 
   Scheduler scheduler;
 
@@ -218,6 +305,8 @@ class Application {
 
   void calculate_looping_components_();
 
+  void feed_wdt_arch_();
+
   std::vector<Component *> components_{};
   std::vector<Component *> looping_components_{};
 
@@ -227,6 +316,9 @@ class Application {
 #ifdef USE_SWITCH
   std::vector<switch_::Switch *> switches_{};
 #endif
+#ifdef USE_BUTTON
+  std::vector<button::Button *> buttons_{};
+#endif
 #ifdef USE_SENSOR
   std::vector<sensor::Sensor *> sensors_{};
 #endif
@@ -234,7 +326,7 @@ class Application {
   std::vector<text_sensor::TextSensor *> text_sensors_{};
 #endif
 #ifdef USE_FAN
-  std::vector<fan::FanState *> fans_{};
+  std::vector<fan::Fan *> fans_{};
 #endif
 #ifdef USE_COVER
   std::vector<cover::Cover *> covers_{};
@@ -245,17 +337,31 @@ class Application {
 #ifdef USE_LIGHT
   std::vector<light::LightState *> lights_{};
 #endif
+#ifdef USE_NUMBER
+  std::vector<number::Number *> numbers_{};
+#endif
+#ifdef USE_SELECT
+  std::vector<select::Select *> selects_{};
+#endif
+#ifdef USE_LOCK
+  std::vector<lock::Lock *> locks_{};
+#endif
+#ifdef USE_MEDIA_PLAYER
+  std::vector<media_player::MediaPlayer *> media_players_{};
+#endif
 
   std::string name_;
+  std::string friendly_name_;
+  std::string comment_;
   std::string compilation_time_;
   bool name_add_mac_suffix_;
   uint32_t last_loop_{0};
   uint32_t loop_interval_{16};
-  int dump_config_at_{-1};
+  size_t dump_config_at_{SIZE_MAX};
   uint32_t app_state_{0};
 };
 
 /// Global storage of Application pointer - only one Application can exist.
-extern Application App;
+extern Application App;  // NOLINT(cppcoreguidelines-avoid-non-const-global-variables)
 
 }  // namespace esphome
