@@ -6,10 +6,9 @@
 
 #ifdef USE_STORE_LOG_STR_IN_FLASH
 #include "WString.h"
+#include "esphome/core/defines.h"  // for USE_ARDUINO_VERSION_CODE
 #endif
 
-#include "esphome/core/macros.h"
-
 // Include ESP-IDF/Arduino based logging methods here so they don't undefine ours later
 #if defined(USE_ESP32_FRAMEWORK_ARDUINO) || defined(USE_ESP_IDF)
 #include <esp_err.h>
@@ -19,8 +18,6 @@
 #include <esp32-hal-log.h>
 #endif
 
-#include "esphome/core/macros.h"
-
 namespace esphome {
 
 #define ESPHOME_LOG_LEVEL_NONE 0
@@ -147,19 +144,12 @@ int esp_idf_log_vprintf_(const char *format, va_list args);  // NOLINT
 #endif
 
 #define ESP_LOGE(tag, ...) esph_log_e(tag, __VA_ARGS__)
-#define LOG_E(tag, ...) ESP_LOGE(tag, __VA__ARGS__)
 #define ESP_LOGW(tag, ...) esph_log_w(tag, __VA_ARGS__)
-#define LOG_W(tag, ...) ESP_LOGW(tag, __VA__ARGS__)
 #define ESP_LOGI(tag, ...) esph_log_i(tag, __VA_ARGS__)
-#define LOG_I(tag, ...) ESP_LOGI(tag, __VA__ARGS__)
 #define ESP_LOGD(tag, ...) esph_log_d(tag, __VA_ARGS__)
-#define LOG_D(tag, ...) ESP_LOGD(tag, __VA__ARGS__)
 #define ESP_LOGCONFIG(tag, ...) esph_log_config(tag, __VA_ARGS__)
-#define LOG_CONFIG(tag, ...) ESP_LOGCONFIG(tag, __VA__ARGS__)
 #define ESP_LOGV(tag, ...) esph_log_v(tag, __VA_ARGS__)
-#define LOG_V(tag, ...) ESP_LOGV(tag, __VA__ARGS__)
 #define ESP_LOGVV(tag, ...) esph_log_vv(tag, __VA_ARGS__)
-#define LOG_VV(tag, ...) ESP_LOGVV(tag, __VA__ARGS__)
 
 #define BYTE_TO_BINARY_PATTERN "%c%c%c%c%c%c%c%c"
 #define BYTE_TO_BINARY(byte) \
@@ -176,7 +166,7 @@ struct LogString;
 
 #include <pgmspace.h>
 
-#if ARDUINO_VERSION_CODE >= VERSION_CODE(2, 5, 0)
+#if USE_ARDUINO_VERSION_CODE >= VERSION_CODE(2, 5, 0)
 #define LOG_STR_ARG(s) ((PGM_P)(s))
 #else
 // Pre-Arduino 2.5, we can't pass a PSTR() to printf(). Emulate support by copying the message to a
